// Copyright Marcus Del Favero 2025
// Licensed under the GNU AGPLv3 with an exception, see `README.md` for details
use super::{ConnectionListener, IncomingConnection, DynIncomingConnection};

use crate::net::message_stream::{DynMs, singleplayer::LocalMessageStream};

pub struct SingleplayerConnectionListener(Option<SingleplayerIncoming>);

impl SingleplayerConnectionListener {
	pub fn new(stream: LocalMessageStream, spectate: bool) -> SingleplayerConnectionListener {
		SingleplayerConnectionListener(Some(SingleplayerIncoming(stream, spectate)))
	}
}

impl ConnectionListener for SingleplayerConnectionListener {
	fn get(&mut self) -> Result<Option<Box<DynIncomingConnection>>, ()> {
		Ok(self.0.take().map(|x| -> Box<DynIncomingConnection> { Box::new(x) }))
	}
}

struct SingleplayerIncoming(LocalMessageStream, bool);

impl IncomingConnection for SingleplayerIncoming {
	fn accept(self: Box<Self>) -> Result<Box<DynMs>, (String, String)> { Ok(Box::new(self.0)) }
	fn reject(self: Box<Self>, _reason: String) {}

	fn spectate_requested(&self) -> bool { self.1 }
}
