// Copyright Marcus Del Favero 2025
// Licensed under the GNU AGPLv3 with an exception, see `README.md` for details
use glam::Vec2;

use super::super::player::{Player, DPlayer, PlayerController, direction::Direction};

use crate::playing::view::HealthBar;
use crate::utils::maths::Circle;

pub struct LocalPlayer {
	pub(in crate::world) player: Player,
	pub(in crate::world) last_hurt: f32,
	pub(in crate::world) hbar: HealthBar,
	pub(super) delta: DPlayer,
}

impl LocalPlayer {
	pub(super) fn new(player: Player) -> LocalPlayer {
		let hbar = HealthBar::new(player.get_norm_health());
		LocalPlayer { player, last_hurt: f32::INFINITY, hbar, delta: DPlayer::zero() }
	}

	pub fn inner(&self) -> &Player { &self.player }

	pub fn get_pos(&self, time_diff: f32) -> Vec2 {
		let pos = self.player.get_pos() + self.delta.get_pos();
		let vel = self.get_vel();
		pos + vel * time_diff
	}

	pub fn get_vel(&self) -> Vec2 { self.player.get_vel() + self.delta.get_vel() }
	pub fn get_dir(&self) -> Direction { self.player.get_dir() + self.delta.get_angle() }
	pub fn get_ammo_string(&self) -> String { self.player.get_ammo_count().to_string() }
	pub fn control(&mut self, pcontrol: &PlayerController) { self.player.set_update(pcontrol.get_update()); }
}
